import { of } from 'rxjs';
import { map, catchError, exhaustMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { AuthResponse } from '@ucap/protocol-query';

import { QueryProtocolService } from '@ucap/ng-protocol-query';
import { OptionProtocolService } from '@ucap/ng-protocol-option';

import {
  auth,
  authSuccess,
  authFailure,
  regView,
  regViewSuccess,
  regViewFailure
} from './actions';
import { RegViewResponse } from '@ucap/protocol-option';

@Injectable()
export class Effects {
  auth$ = createEffect(() =>
    this.actions$.pipe(
      ofType(auth),
      map(action => action.req),
      exhaustMap(req => {
        return this.queryProtocolService.auth(req).pipe(
          map((res: AuthResponse) => {
            return authSuccess({
              res
            });
          }),
          catchError(error => of(authFailure({ error })))
        );
      })
    )
  );

  regView$ = createEffect(() =>
    this.actions$.pipe(
      ofType(regView),
      map(action => action.req),
      exhaustMap(req => {
        return this.optionProtocolService.regView(req).pipe(
          map((res: RegViewResponse) => {
            return regViewSuccess({
              res
            });
          }),
          catchError(error => of(regViewFailure({ error })))
        );
      })
    )
  );

  constructor(
    private actions$: Actions,
    private queryProtocolService: QueryProtocolService,
    private optionProtocolService: OptionProtocolService
  ) {}
}
